#define LINUX_VERSION_CODE 265942
#define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
#define AMZN_MAJOR 2
#define AMZN_MINOR 0
#define AMZN_RELEASE_VERSION(a,b) (((a) << 8) + (b))
#define AMZN_RELEASE_CODE 512
